//
// these eight functions draw straight-line dark (move) and
// light (line) vectors between one point and another on the
// graphics screen.
//
#include "gogo.h"
//
// moves (draws a dark vector) to (x, y),
// where x and y are user units.
//

void M(flt x, flt y)
{
    vector<flt> u (3);  // user coords of point
    vector<int> g (2);  // graphics screen coords of point

    u[0] = x;
    u[1] = y;
    u[2] = 1.0;      // shifts origin (according to trm)

    g = u2g(u);      // transforms u to g
    MG(g);           // moves to (g[0], g[1]) if this point is
                     // inside window; otherwise, does nothing.
}
// ========================================================
//
// draws a line (vector) to (x, y),
// where x and y are user units.
//

void L(flt x, flt y)
{
    vector<flt> u (3); // user coordinates of point
    vector<int> g (2); // graphics screen coordinates of point

    u[0] = x;
    u[1] = y;
    u[2] = 1.0;       // shifts origin

    g = u2g(u);       // transforms u to g
    LG(g);            // draws line to (g[0], g[1]) if this point
                      // AND the previous point were inside window;
                      // if this point is inside, and the previous
                      // point was outside, just MOVES to this point;
                      // otherwise, does nothing.
}
// ========================================================
//
// moves (draws a dark vector) to graph g if
// this point is inside the window
//

void MG(vector<int>& g)
{
    glast[0] = g[0];
    glast[1] = g[1];

    if (g[0] < w[0] || g[0] > w[2] || g[1] < w[1] || g[1]> w[3]){
        oob = true;  // (g[0], g[1]) is outside the window
    }
    else {           // (g[0], g[1]) is inside the window
        oob = false;
        moveto(glast);
    }
}
//
// moves (draws a dark vector) to (xg, yg) if
// this point is inside the window; this is like the
// older version
//

void MG(int xg, int yg)
{
    glast[0] = xg;
    glast[1] = yg;

    if (xg < w[0] || xg > w[2] || yg < w[1] || yg> w[3]){
        oob = true;  // (xg, yg) is outside the window
    }
    else {           // (xg, yg) is inside the window
        oob = false;
        moveto(glast);
    }
}
// ========================================================
//
// draws a line (vector) to the coords of g if
// this point is inside the window and
// previous point was inside the window
//

void LG(vector<int>& g)
{
    glast[0] = g[0];
    glast[1] = g[1];

    if (g[0] < w[0] || g[0] > w[2] || g[1] < w[1] || g[1] > w[3]){
        oob = true;  // (g[0], g[1]) is outside the window
    }
    else {           // (g[0], g[1]) is inside the window
        if (oob){    // previous point was outside
            moveto(glast);
        }
        else {       // previous point was inside
            lineto(glast);
        }
        oob = false;
    }
}
//
// draws a line (vector) to (xg, yg) if
// this point is inside the window and
// previous point was inside the window
// this is like the older version of LG
//

void LG(int xg, int yg)
{
    glast[0] = xg;
    glast[1] = yg;

    if (xg < w[0] || xg > w[2] || yg < w[1] || yg > w[3]){
        oob = true;  // (xg, yg) is outside the window
    }
    else {           // (xg, yg) is inside the window
        if (oob){    // previous point was outside
            moveto(glast);
        }
        else {       // previous point was inside
            lineto(glast);
        }
        oob = false;
    }
}
// ========================================================
//
// moves relative to the current (most recent)
// draw point (xg, yg), to (xg + rxg, yg + ryg).
//

void RMG(int rxg, int ryg)
{
    vector<int> g(2);
    g[0] = glast[0] + rxg;
    g[1] = glast[1] + ryg;
    MG(g);
}
// ========================================================
//
// draws a line (vector) relative to the current (most
// recent) draw point (xg, yg), to (xg + rxg, yg + ryg).
//

void RLG(int rxg, int ryg)
{
    vector<int> g (2);
    g[0] = glast[0] + rxg;
    g[1] = glast[1] + ryg;
    LG(g);
}
// ========================================================
//
// moves (draws a dark vector) to (g[0], g[1])
// this primitive is generally called by other
// drawing functions.
//

void moveto(vector<int>& g)
{
    char yhi, ylo, xhi, xlo;

    yhi = 0x20 | g[1] >> 5 & 0x1f;
    ylo = 0x60 | g[1] & 0x1f;
    xhi = 0x20 | g[0] >> 5 & 0x1f;
    xlo = 0x40 | g[0] & 0x1f;
    gassert(ofs);
    out(ofs, '\035', yhi, ylo, xhi, xlo); // \035: ctrl-]
                                          // graph mode pen up?
}
// ========================================================
//
// draws a line (vector) to (g[0], g[1])
// this primitive is generally called by other
// drawing functions.
//

void lineto(vector<int>& g)
{
    char yhi, ylo, xhi, xlo;

    yhi = 0x20 | g[1] >> 5 & 0x1f;
    ylo = 0x60 | g[1] & 0x1f;
    xhi = 0x20 | g[0] >> 5 & 0x1f;
    xlo = 0x40 | g[0] & 0x1f;

    gassert(ofs);
    out(ofs, yhi, ylo, xhi, xlo);
}
// ========================================================


