// prints a brief "help file" on the screen
//

#include "lorenz.h"

void prn_info(char** argv)
{
    string pgname = basename(argv[0]);  // chops long pathname
    prn_usage(argv);

    cout <<
        "  INFORMATION:\n\n"
        "   \033[1;32m" << pgname << "\033[0m, when run from an \033[1;33mxterm\033[0m window, will display, in the xterm's\n"
        "   \033[1;33mtektronix(Tek)\033[0m window, a real-time plotting of the Lorenz attractor.\n"
        "   To ensure a reliable display of the attractor, certain key xterm resources\n"
        "   (typically inserted in a file called \033[1;33m.Xresources\033[0m in your home directory)\n"
        "   are required.  Here is a suggested list:\n\n"
        "    \033[1;33mxterm*utf8: 0\n"
        "    xterm*Tek4014.ginTerminator: CRonly\n"
        "    xterm*background: #332211\n"
        "    xterm*foreground: #ffffff\n"
        "    xterm*faceName: DejaVu Sans Mono\n" 
        "    xterm*faceSize: 9\n"
        "    VT100*geometry: 80x50\n"
        "    xterm*charClass: 33:48,36-47:48,58-59:48,61:48,63-64:48,95:48,126:48\n"
        "    *VT100*translations: #override Meta <Btn1Up>: \\\n"
        "          exec-formatted(\"firefox \'%t\'\", PRIMARY)\033[0m\n\n"
        "   The first two are required. The rest are optional, and may be chosen to suit\n"
        "   your taste.\n\n"
        "   Here the background will be dark brown and the foreground will be white.\n"
        "   The faceName font resource sets a popular font for the xterm.  The last two\n"
        "   resources allow (for xterm versions >= 277) URLs displayed in an xterm to\n"
        "   be clickable; replace the word firefox with the name of your default browser\n"
        "   if necessary.  To bring up http://wikipedia.org/ (for example),\n"
        "   (a) double-click on the URL to select it, then (b) ALT-click anywhere on the\n"
        "   screen to display the web page pointed to by the URL.\n\n"
        "   After you've edited your \033[1;33m.Xresources\033[0m file, you need to ensure\n"
        "   that your new xterm resources are used, so give the command\n"
        "    \033[1;32mxrdb -merge ~/.Xresources\033[0m\n\n"
        "   To run the program, simply give the command \033[1;32m" << pgname << "\033[0m.\n\n"
        "   When the program starts, it asks you to input a blocksize.  This parameter\n"
        "   governs the number of points plotted before stopping to read any key\n"
        "   presses you might have made.  Larger values will allow faster plotting,\n"
        "   but if it\'s too large, it will take a while to respond to your key presses.\n"
        "   If the value is too small (like 1 or 2) the program will spend too much time\n"
        "   looking for key presses, and not much time plotting.  The default value of\n"
        "   50 is reasonable.  To use it, just hit the ENTER key.  Then the Tek window\n"
        "   should pop up and the graph of the attractor should start to be displayed.\n\n"
        "   NOTE:  On starting the program, if you happen to get endless screenfuls\n"
        "   of weird characters, it probably means you have not set the needed\n"
        "   X resources correctly.  Just hit \033[1;33mx\033[0m (or \033[1;33mq\033[0m), and check to see that they\n"
        "   are correctly set.  The command \033[1;32mxrdb -query -all\033[0m will list the X resources\n"
        "   currently set.\n\n"
        "   While the plotting takes place, certain keys are \"hot\", such that pressing\n"
        "   them will cause certain things to happen.  Here is a complete list:\n\n"
        "    \'\033[1;33mx\033[0m\' or \'\033[1;33mq\033[0m\' will quit the program.\n"
        "    \'\033[1;33mr\033[0m\' will decrease the value of the Rayleigh number r.\n"
        "    \'\033[1;33mR\033[0m\' will increase the value of the Rayleigh number r.\n"
        "    \'\033[1;33mb\033[0m\' will decrease the value of the viewing angle beta.\n"
        "    \'\033[1;33mB\033[0m\' will increase the value of the viewing angle beta.\n"
        "    \'\033[1;33mc\033[0m\' will clear the screen and start a new plot.\n"
        "    \'\033[1;33mi\033[0m\' will allow you to input a new starting point.  When the cursor changes\n"
        "      to a large plus sign, just move the mouse to the desired starting\n"
        "      point and click any mouse button or any letter key.  The new starting\n"
        "      point coordinates will be displayed in the normal xterm window.\n\n"
        "  BACKGROUND NOTES:\n\n"
        "   The Lorenz Equations were introduced by Edward N. Lorenz in his\n"
        "   seminal paper, \"Deterministic Nonperiodic Flow\", published in the\n"
        "   Journal of Atmospheric Sciences, Vol. 20, March 1963, page 130.  This paper\n"
        "   is believed to be the most frequently cited paper in the chaos literature.\n"
        "   A copy of this paper may be downloaded from\n"
        "   http://sirius.ucsc.edu/users/drip/lorenz_1963.pdf\n\n"
        "   The equations---three first order coupled nonlinear differential\n"
        "   equations---describe the flow patterns in a layer of fluid of uniform depth\n"
        "   that is heated from below and cooled at the top, with a constant\n"
        "   temperature difference \033[1;33mDelta-T\033[0m between the bottom and the top of the layer.\n"
        "   They are a simplification of the Navier-Stokes equations.  The resulting\n"
        "   convection pattern is known as Rayleigh-B\351nard convection.\n\n"
        "   Lorenz was a meteorologist, thinking that such a layer might be a simple\n"
        "   model for a layer of the Earth's atmosphere, so the behavior of the\n"
        "   solutions might shed some light on how predictable the weather might be.\n\n"
        "   If \033[1;33mDelta-T\033[0m is small, heat flow through the fluid occurs by conduction only,\n"
        "   with the temperature varying linearly between the bottom and the top.  In\n"
        "   this case, there is no motion of the fluid.\n\n"
        "   If \033[1;33mDelta-T\033[0m is greater than some critical value, fluid convection will\n"
        "   occur, with pairs of fluid \"rolls\" forming in the fluid layer that serve\n"
        "   to transfer heat from the bottom to the top of the layer.\n\n"
        "   If \033[1;33mDelta-T\033[0m is greater than a still larger critical value, the circulation\n"
        "   of a \"roll\" will itself become unstable, with circulation occuring first\n"
        "   in one direction, then coming to a halt and reversing direction, with a\n"
        "   pattern of repetition that is nonperiodic, never repeats, and is not\n"
        "   predictable.\n\n"
        "   Here are the equations, as presented by Lorenz:\n\n"
        "     \033[1;33mdX/dt = -sigma*X  + sigma*Y          (25)\033[0m\n"
        "     \033[1;33mdY/dt = -X*Z + r*X -Y                (26)\033[0m\n"
        "     \033[1;33mdZ/dt = X*Y  - b*Z                   (27)\033[0m\n\n"
        "   where \033[1;33mt\033[0m is a dimensionless time variable.\n\n"
        "   The three parameters are (a) the Prandtl number \033[1;33msigma\033[0m, which is the ratio\n"
        "   of the kinematic viscosity to the thermal conductivity, taken by Lorenz\n"
        "   to equal 10; (b) \033[1;33mr\033[0m, the ratio of the Rayleigh number \033[1;33mR\033[0m (which is\n"
        "   proportional to \033[1;33mDelta-T\033[0m) to its critical value; and (c) \033[1;33mb\033[0m, a number\n"
        "   related to the \"aspect ratio\", or the geometry of the fluid layer, taken\n"
        "   by Lorenz to be 8/3---its value when the critical value of the Rayleigh\n"
        "   number is at its minimum.  For the given values of \033[1;33msigma\033[0m and \033[1;33mb\033[0m, the\n"
        "   critical value of \033[1;33mr\033[0m, above which convection becomes unstable, is\n"
        "   470/19 = 24.74.\n\n"
        "   The three variables are \033[1;33mX\033[0m, which is proportional to the intensity of the\n"
        "   convection (so that \033[1;33mX > 0\033[0m means fluid is ascending while \033[1;33mX < 0\033[0m means fluid\n"
        "   is descending at some position in the layer); \033[1;33mY\033[0m, which is proportional to\n"
        "   the temperature difference between the ascending and descending currents\n"
        "   (similar signs of \033[1;33mX\033[0m and \033[1;33mY\033[0m means warm fluid is rising and cold fluid is\n"
        "   descending); and \033[1;33mZ\033[0m, which is proportional to the distortion of the vertical\n"
        "   temperature profile from linearity, a positive value of \033[1;33mZ\033[0m indicating that\n"
        "   the strongest gradients appear near the boundaries of the layer.\n\n"
        "   The above equations have a steady solution (fixed point) for \033[1;33mX = Y = Z = 0\033[0m,\n"
        "   representing a state of no convection.  This state becomes unstable for\n"
        "   \033[1;33mr > 1\033[0m so convection will occur when \033[1;33mr > 1\033[0m.\n\n"
        "   In this situation two new fixed points appear at \033[1;33mX = Y = +/- [b(r-1)]^0.5\033[0m,\n"
        "   \033[1;33mZ = r-1\033[0m, and these two points become unstable if \033[1;33mr\033[0m is sufficiently large,\n"
        "   that is, if \033[1;33mr > sigma(sigma+b+3)/(sigma-b-1)\033[0m, or for the given values of\n"
        "   \033[1;33msigma\033[0m and \033[1;33mb\033[0m, if \033[1;33mr > 470/19 = 24.74\033[0m.\n\n"
        "   For the purposes of our program, we have scaled the above equations so as\n"
        "   to make the coordinates of the fixed points (and hence the layout of our\n"
        "   graphical display) independent of the parameter values, at \033[1;33mX = Y = +/- 1\033[0m,\n"
        "   \033[1;33mZ = 1\033[0m.\n\n"
        "   Thus here are the equations that are solved by this computer program:\n\n"
        "     \033[1;33mdx/dt = -sigma*x   + sigma*y\033[0m\n"
        "     \033[1;33mdy/dt = -(r-1)*x*z + r*x - y\033[0m\n"
        "     \033[1;33mdz/dt = b*x*y - b*z\033[0m\n\n"
        "   where \033[1;33msigma\033[0m, \033[1;33mr\033[0m and \033[1;33mb\033[0m are identical to those used by Lorenz.\n\n"
        "   We use a \"quality controlled\" 4th order Runge-Kutta routine to solve these\n"
        "   equations, given some starting set of values for \033[1;33mx\033[0m, \033[1;33my\033[0m and \033[1;33mz\033[0m.\n\n"
        "  MORE BACKGROUND NOTES:\n\n"
        "   For those interested, additional information may be found on this Wikipedia\n"
        "   page: http://en.wikipedia.org/wiki/Lorenz_system .  This page features a\n"
        "   number of sophisticated animations of the Lorenz attractor, in addition to\n"
        "   reviewing the history of the development of the equations by Lorenz.\n\n"
        "   Finally, in an interesting parallel, the Lorenz Equations also describe\n"
        "   the behavior of a special kind of water wheel, first constructed by Willem\n"
        "   Malkus and Lou Howard at MIT in the 1970s.  A web page describing this\n"
        "   water wheel (now called the \"Lorenz-Malkus\" water wheel), including an\n"
        "   historic video showing the original waterwheel in operation, may be found\n"
        "   at http://sirius.ucsc.edu/users/drip/lorenz_waterwheel.php\n\n"
        "   Feel free to write to me if you have comments, or if something does not\n"
        "   work for you:  Peter Scott, mailto:drip@ucsc.edu\n\n"
        "   To facilitate your reading of this help file, try \033[1;32mlorenz -h | more\033[0m\n\n";
}
